import time
from typing import Dict

# time reporting helper
class TimeRep:

    def __init__(self):
        self.tr: Dict[str,float] = {}
        self.stime: float = time.time()
        self.stime_init = self.stime

    # resets and starts
    def start_now(self):
        self.tr = {}
        self.stime = time.time()

    def log(self, past_interval_name:str):
        ct = time.time()
        self.tr[past_interval_name] = ct - self.stime
        self.stime = ct

    def get_report(self) -> Dict[str,float]:
        rep = {}
        rep.update(self.tr)
        return rep

    def __str__(self):
        rep = self.get_report()
        rep['___total time'] = time.time() - self.stime_init
        return "\n".join([f'{k:30}: {v:9.3f}s' for k,v in rep.items()])


# normalizes whitespace characters & newlines -> single space
def whitespace_normalization(
        text: str,
        remove_nlines=  True) -> str:  # for False leaves single (space separated) '\n' between lines, for True replaces '\n' with space

    def _line_whitespace_normalization(txt: str) -> str:
        if txt: txt = ' '.join(txt.split())
        return txt

    lines = [text] if remove_nlines else text.split('\n')
    lines = [_line_whitespace_normalization(txt=line) for line in lines]
    lines = [l for l in lines if l]  # remove empty lines
    return '\n'.join(lines)